/*
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
| File Name			  : MicroBoltDebuggerExample.c																					
| Author			    : Micromint, Inc.						
| Copyright       : Copyright  2005, Micromint, Inc.																	
| Creation Date		: 4/2/05																							
| Version			    : 1.00																																												
| Spaces per tab	:	2																									
| Description		  : Main C file																																									 
| Revision			  :	Initial																																																						
|------------------------------------------------------------------------------------------
|------------------------------------------------------------------------------------------
*/

/*
|----------------------------------------------------
|     Includes
|----------------------------------------------------
*/

#include <ARM/philips/lpc210x.h>
#include <arm_macros.h>
#include <STDIO.h>

#include "MicroBoltDebuggerExample.h"

/*
|------------------------------------------------------------------------------------------
|     Function        :   main
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Main function for system
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

extern int _textmode;		                        // This is defined in the ICCARM library 
unsigned char TestLedFlag = LED_BLINK;
		
void main(void)
  {

/*
|----------------------------------------------------
|     MicroBolt hardware setup 
|----------------------------------------------------
*/

  unsigned int Delay; 
  unsigned int Count = 0;

  __DISABLE_INTERRUPT();                        // Disable all interrupts

/*
|-------------------------
| Config MAM 
|-------------------------
*/
  MAM_CR = 0x00;                                // Turn MAM off (default)
  MAM_TIM = 0x04;                               // Set flash timing to 4 clock cycles

  MAM_CR = 0x02;                                // Fully enable the Memory Accleration Module

/*
|-------------------------
| Config PLL 
|-------------------------
*/
  VICVectAddr8 = (unsigned)pll_isr;             // Assign the PLL lock ISR vector address
  VICVectCntl8 = INTERRUPT_CHANNEL_FOR_PLL;     // Assign the VIC address to the actual interrupt
  VICIntEnable = INTERRUPT_ENABLE_FOR_PLL;      // Enable the interrupt  

  SCB_PLLCFG |= 0x23;				 	   		            // Set to 59 MHz (0x03 is multiply value of 4)
  SCB_PLLCON |= 0x01;                           // Enable the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG

/*
|-------------------------
| Config GPIO 
|-------------------------
*/
  PCB_PINSEL0=0x00000000;                       // JTAG is via secondary port
  PCB_PINSEL1=0x55400000;
 
  GPIO_IODIR=(0x00000000<<16)|                  // Make all inputs to start with
	     0x00000000;

  GPIO_IODIR |= MICROBOLT_LED;                  // Setup MicroBolt LED as output

/*
|-------------------------
| Config UART-O 
|-------------------------
*/
  PCB_PINSEL0 |= P0_0_UART_0_TX;                // Setup P0.0 to alternate function UART0-TX
  PCB_PINSEL0 |= P0_1_UART_0_RX;                // Setup P0.1 to alternate function UART0-RX

  VICVectAddr0 = (unsigned)Uart0_ISR;           // Assign the UART-0 ISR vector address
  VICVectCntl0 = 0x20 | 6;                      // Assign the VIC address to the actual interrupt
  VICIntEnable = 0x00000040;                    // Enable the interrupt  

  UART0_IER = 0x00000001;	 	                   // Receive interrupts
  UART0_FCR = 0x00000001;	 	                   // Enable the fifos
  UART0_LCR = 0x00000083;	 	                   // Enable the divisor
  UART0_DLM = 0;				                       // Divisor latch MSB (for baud rates < 4800) 
  UART0_DLL = BAUD_RATE_115200;			           // Divisor latch LSB 
  UART0_LCR = 0x00000003;  	                   // Close it, then UART works with divisor

  __ENABLE_INTERRUPT();                         // Enable all interrupts

/*
|----------------------------------------------------
|     Start of application
|----------------------------------------------------
*/

  while(1)                                            // Do this forever
    {
    Count++;                                          // Increment test counter
                                                    
    printf("1The count starts here --> %d\n", Count); // Output string to Debug window 1
    printf("2%d\n", Count+1);                         // Output string to Debug window 2 
    printf("3%d\n", Count+2);                         // Output string to Debug window 3
    printf("4%d\n", Count+3);                         // Output string to Debug window 4 
    printf("5%d\n", Count+4);                         // Output string to Debug window 5 
    printf("6%d\n", Count+5);                         // Output string to Debug window 6 
    printf("7%d\n", Count+6);                         // Output string to Debug window 7 
    printf("8And finally ends here...%d\n", Count+8); // Output string to Debug window 8
    printf("9Hello World!\n");                        // Output string to Debug window 9 

    if (TestLedFlag == LED_BLINK)
      {
      GPIO_IOSET = MICROBOLT_LED;                       // MicroBolt LED On
      for (Delay = 0; Delay < 600000; Delay++);         // Delay
      printf("L1\n");
      GPIO_IOCLR = MICROBOLT_LED;                       // MicroBolt LED Off
      for (Delay = 0; Delay < 600000; Delay++);         // Delay
      printf("L0\n");
      }
    else
    if (TestLedFlag == LED_ON)
      {
      GPIO_IOSET = MICROBOLT_LED;                       // MicroBolt LED On
      for (Delay = 0; Delay < 600000; Delay++);         // Delay
      printf("L1\n");
      }
    else
    if (TestLedFlag == LED_OFF)
      {
      GPIO_IOCLR = MICROBOLT_LED;                       // MicroBolt LED Off
      for (Delay = 0; Delay < 600000; Delay++);         // Delay
      printf("L0\n");
      }              
    }
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   putchar
|     Inputs          :   Byte for transmission
|     Outputs         :   None
|     Purpose         :   Transmit a byte via UART0 to enable printf
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

int putchar(char SerialByte)
  {
  _textmode = 1;                                      // Turn carriage return line feed on for printf
  if (_textmode == 1 && SerialByte == '\n')
      {
      putchar('\r');
      }
  while (!(UART0_LSR & 0x20)); 
  UART0_THR = SerialByte;
  return SerialByte;
  }


/*
|------------------------------------------------------------------------------------------
|     Function        :   pll_isr
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Once PLL has locked, connect it and use for system clock
|     Author          :   Micromint, Inc.
|------------------------------------------------------------------------------------------
*/

#pragma interrupt_handler pll_isr
void pll_isr(void)
  {
  SCB_PLLCON |= 0x02;                           // Connect the PLL 
  SCB_PLLFEED = 0xAA;					   	  	          // Shadow register copy to enable changes
  SCB_PLLFEED = 0x55;                           // in PLLCON and PLLCFG
  VICIntEnClear = PLL_CLR;                      // Disable the PLL interrupt, not needed anymore
  VICVectAddr = VIC_ACK;                        // Acknowledge Interrupt
  }

/*
|------------------------------------------------------------------------------------------
|     Function        :   Uart0_ISR
|     Inputs          :   None
|     Outputs         :   None
|     Purpose         :   Interrut service routine for Uart-0
|     Author          :   Pride Embedded, LLC
|------------------------------------------------------------------------------------------
*/

#pragma interrupt_handler Uart0_ISR

void Uart0_ISR (void)   
  {
  static unsigned char CharRxCount = 0;
  static unsigned char SerialBuffer[20];   
  static unsigned char LedFlip; 
  
  switch(UART0_IIR & 0x0F)
      {
      case 0x04:                                                  // Receive data available

/*
|----------------------------------------------------
|     Get serial port data and check for commands
|----------------------------------------------------
*/            
      SerialBuffer[CharRxCount] = UART0_RBR;

      if (SerialBuffer[0] == 'L')
          {
          if (CharRxCount == 2)
              {
              switch(SerialBuffer[1])
                  {
                  case 'A':  
                      {
                      TestLedFlag = LED_ON;                                                 
                      break;
                      }
                  case 'B':  
                      {
                      TestLedFlag = LED_BLINK;                                                 
                      break;
                      }
                  case 'C':  
                      {
                      TestLedFlag = LED_OFF;                                                 
                      break;
                      }
                  default:
                      {
                      break;
                      }
                  }
              CharRxCount = 0; 
              }
          else
              {
              CharRxCount++;      
              }          
          }
    else
          {
          CharRxCount = 0;        
          }          
    break;

    default:
    break;
    }
	  VICVectAddr = 0;                            // Acknowledge Interrupt
  }